<?
/***************************************************************
*  Copyright notice
*
*  (c) 2005 Rainer Sudhoelter <r.sudhoelter (at) web.de>
*  All rights reserved
*
*  This script is part of the TYPO3 project. The TYPO3 project is
*  free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  The GNU General Public License can be found at
*  http://www.gnu.org/copyleft/gpl.html.
*
*  This script is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/

/**
 * Module 'import' for the 'rs_userimp' extension.
 *
 * @author	Rainer Sudhoelter <r.sudhoelter (at) web.de>
 */
 
/**
 * [CLASS/FUNCTION INDEX of SCRIPT]
 *
 *
 *
 *   81: class tx_rsuserimp
 *
 *              SECTION: Section 1
 *  121:     function init()
 *  212:     function start()
 *
 *              SECTION: Section 2
 *  234:     function readCSV()
 *  257:     function readSamplesFromCSV()
 *  285:     function getColumnNamesFromCSV()
 *  307:     function getColumnNamesFromDB()
 *  327:     function getExtraFieldsFromDB()
 *  353:     function removeNoMapFields($dbFields)
 *  370:     function createImportForm()
 *  387:     function createMappingForm ()
 *  433:     function evaluateMappingForm ()
 *  519:     function createSelector($x)
 *  552:     function fieldSelector($n)
 *
 *              SECTION: Section 3
 *  583:     function importUsers ()
 *  743:     function checkUserDataFE (&$user,&$importUser)
 *  859:     function checkUserDataBE (&$user,$importUser)
 *  877:     function _fputcsv($fileName, $dataArray, $delimiter, $enclosure)
 *
 * TOTAL FUNCTIONS: 17
 * (This index is automatically created/updated by the extension "extdeveval")
 *
 */

/**
 * This class provides CSV to TYPO3 DB user import features. It lets you map CSV to
 * corresponding user-specific DB fields and was inspired by the need to mass-import >> 1000 FE users.
 * Mapping features have successfully applied on Microsoft (c) Outlook Express WAB and Outlook address books,
 * CSV exports of Microsoft (c) Excel sheets and general text export files from 3COM (c) Palm Desktop and Lotus Notes (c).
 * The class is to be called after all necessary pre-mapping info has been provided.
 * This means we already have a CSV file in place and basic import setting configuration.
 *
 * USE:
 * The class is intended to be used by creating an instance of it.
 *
 * @author	Rainer Sudhoelter
 * @package rs_userimp
 */

 require_once(PATH_t3lib.'class.t3lib_div.php');

class tx_rsuserimp {

	var $previewNum;
	var $CSVhasTitle;
	var $fieldDelimiter;
	var $fieldEncaps;
	var $fieldmap;
	var $mandatoryFields;
	var $additionalMandatoryFieldsFE;
	var $mandatoryFieldsBE;
	var $defaultUserData;
	var $file;
	var $CSV = array();
	var $num;
	var $importOK;
	var $importNow;
	var $map;
	var $columnNamesFromCSV;
	var $columnNamesFromDB;
	var $numMap;
	var $inData = array();
	var $enableAutoRename;
	var $userType;
	var $useRecycler;
	var $createDropFile;

	/*************************
	 *
	 * Section 1
	 *
	 * Explanation:
	 * Initialization of class.
	 *
	 *************************/

	/**
	 * Pseudo Constructor: set default import values
	 *
	 * @return	void		initialize the class
	 */
	function init() {

		$this->userType = 'BE';

		$this->enableAutoRename = FALSE;

		$this->bg1 = '#FFEFBF';
		$this->bg2 = '#FFE79F';

		$this->mandatoryFields = array(
			'username',
			'password');
	/**
	 * These fields are removed from the mapping process since they are either computed
	 * or are set once the user logs in for the first time ...
	 */
		$this->noMapFE = array(
			'TSconfig',
			'crdate',
			'crdate',
			'cruser_id',
			'cruser_id',
			'deleted',
			'disable',
			'endtime',
			'fe_cruser_id',
			'image',
			'image',
			'is_online',
			'lastlogin',
			'lockToDomain',
			'pid',
			'starttime',
			'tstamp',
			'uc',
			'uc',
			'uid',
			'usergroup'
			);

	/**
	 * In preparation for BEuserImp ...
	 * These fields are removed from the mapping process since they are either computed
	 * or are set once the user logs in for the first time ...
	 */
		$this->noMapBE = array(
			'TSconfig',
			'admin',
			'allowed_languages',
			'createdByAction',
			'db_mountpoints',
			'disable',
			'disableIPlock',
			'endtime',
			'file_mountpoints',
			'fileoper_perms',
			'lang',
			'lockToDomain',
			'options',
			'starttime',
			'userMods',
//			'email',
//			'password',
//			'realName',
//			'usergroup',
//			'username'
			);

		$this->defaultUserData = array(
			'usergroup' => '',
			'pid' => '',
			'tstamp' => mktime(),
			'crdate' => mktime()
			);

		$this->useRecycler = 1;	// 0 = no, 1 = if available, 2 = always

		$this->previewNum = 3;

		$this->CSVhasTitle = TRUE;

		$this->importNow = $_POST['importNow'] ? 'TRUE' : 'FALSE';

	} //end pseudo constructor

	/**
	 * Get basic mapping data needed for dropdown menu.
	 * Read in sample CSV data to display in mapping session.
	 *
	 * @return	void
	 */
	function start() {

		$this->CSV = $this->readSamplesFromCSV();
		$this->columnNumCSV = count($this->CSV[0]);
		$this->columnNamesFromCSV = $this->getColumnNamesFromCSV();
		$this->columnNamesFromDB = $this->getColumnNamesFromDB();
	}

	/*************************
	 *
	 * Section 2
	 *
	 * Explanation:
	 * File functions: reading CSV files (partly and as a whole)
	 *
	 *************************/

	/**
	 * Read in the given CSV file. The function is used during the final file import.
	 *
	 * @return	array		file content in array
	 */
	function readCSV() {

		$file = $this->file;
		array($mydata);
		$handle = fopen($file, "r");
		$i=0;
		$delimiter = ($this->fieldDelimiter == 'TAB') ? chr(9) : $this->fieldDelimiter ;
		while (($data = fgetcsv($handle, 10000, $delimiter, $this->fieldEncaps)) !== FALSE) {
			$mydata[] = $data;
		}
		fclose ($handle);
		reset ($mydata);
		if ($this->CSVhasTitle) {
			$mydata = array_slice($mydata,1); //delete first row
		}
		return $mydata;
	}

	/**
	 * Read in sample data from CSV file. Needed for the mapping session to display some samples.
	 *
	 * @return	array		sample file content in array	...
	 */
	function readSamplesFromCSV() {

		$file = $this->file;
		array($mydata);
		$handle = fopen($file, "r");
		$i=0;
		$delimiter = ($this->fieldDelimiter == 'TAB') ? chr(9): $this->fieldDelimiter ;
		while (($data = fgetcsv($handle, 10000, $delimiter, $this->fieldEncaps)) !== FALSE) {
			$mydata[] = $data;
			if ($i == $this->previewNum) {
				break;
			} else {
				$i++;
			}
		}
		fclose ($handle);
		reset ($mydata);
		return $mydata;
	}


	/**
	 * Reads the fieldnames from the CSV file. If there are no fieldnames in the first row, create fieldnames
	 * Field[x] (needed for mapping session).
	 * Also cuts off the first array item if is has field titles.
	 *
	 * @return	array
	 */
	function getColumnNamesFromCSV() {

		global $LANG;
		$num = count($this->CSV[0]);

		if ($this->CSVhasTitle) {
			$myheader = $this->CSV[0];
			$this->CSV = array_slice($this->CSV, 1); //delete first row
		} else {
			for ($n = 0; $n < $this->columnNumCSV; $n++) {
				$myheader[$this->CSV[0][$n]] = $LANG->getLL('f1.tab3.mapper.fieldset2.field').sprintf("[%02s]",$n);
			}
		}
		return $myheader;
	}

	/**
	 * Reads DB columns (fieldnames) from the DB using $TCA.
	 * Also, disallowed mapping fields are removed by calling removeNoMapFields(&$fields).
	 *
	 * @return	array
	 */
	function getColumnNamesFromDB() {

		global $TCA;

		/**
		 * This was the initial way of getting available DB table fields for table fe_users:
		 * $dbFields = array_keys($TCA['fe_users']['columns']);
		 * Didn't work on 3.8.0rc1 systems, so it had to be changed to the following
		 */
		$dbFields = explode(',',$TCA['fe_users']['feInterface']['fe_admin_fieldList']);
		$this->removeNoMapFields(&$dbFields); //passed by REFERENCE

		return $dbFields;
	}

	/**
	 * Get additional fields from DB which are not standard and not forbidden by rule.
	 *
	 * @return	array
	 */
	function getExtraFieldsFromDB() {

		global $TCA;

		$dbFieldsDefault = explode(',',$TCA['fe_users']['feInterface']['fe_admin_fieldList']);

		$dbFields = $GLOBALS['TYPO3_DB']->admin_get_fields('fe_users');
		$dbFieldsAll = array();
		foreach ($dbFields as $key=>$value) {
			$dbFieldsAll[] = $key;
		}

		$this->removeNoMapFields(&$dbFieldsAll); //passed by REFERENCE

		$dbFieldsAdditional = array_diff($dbFieldsAll,$dbFieldsDefault);
		reset($dbFieldsAdditional);

		return $dbFieldsAdditional;
	}

	/**
	 * Delete disallowed mapping fields from passed fieldnames.
	 *
	 * @param	array		$dbFields passed by REFERENCE
	 * @return	void
	 */
	function removeNoMapFields($dbFields) {

		$i = 0;
		foreach ($dbFields as $key => $value) {
			if (in_array($value, $this->noMapFE)) {
				unset ($dbFields[$i]);
			}
			$i++;
		}
		$dbFields = array_values($dbFields);
	}

	/**
	 * Displays the import button for the import form.
	 *
	 * @return	string		HTML content
	 */
	function createImportForm() {

		global $LANG;

		$content = $LANG->getLL('f1.tab4.section.import.label').'
				<div align="right">
					<input type="submit" name="importNow" value="'.$LANG->getLL('f1.tab4.section.import.import',1).'" '.($_POST['importNow'] ? 'disabled' : '').' onclick="return confirm(\''.$LANG->getLL('f1.tab4.section.import.sure',1).'\');">
				</div>';
		return $content;
	}


	/**
	 * Display the mapping form.
	 *
	 * @return	string		HTML content		...
	 */
	function createMappingForm () {

		global $LANG;

		$content = '';
		$content .= '<fieldset>';
		$content .= '<legend align=left><b>'.$LANG->getLL('f1.tab3.mapper.fieldset1').'</b></legend>';

		if ($this->inData['fieldname']) {
			$map = array();
			$content .= $this->evaluateMappingForm();
		}

		if (!$this->inData['import']) {
			$content .= '<div align="right"><input type="submit" name="map" value="'.$LANG->getLL('f1.tab3.mapper.import').'" '.($_POST['importNow'] ? 'disabled' : '').'></div>';
			$content .= '<table style="font-size : 8pt;" width="100%" border=0 callpading=0 cellspacing=0>';
			$i = 0;

			$content .= '<tr style="background-color:#FFE79F;">
							<td><b>#</b></td>
							<td><b>Description</b></td>
							<td><b>Mapping</b></td>
							<td><b>Example values from CSV</b></td>
							</tr>';

			foreach ($this->columnNamesFromCSV as $key) {
				$colorcount = ($colorcount == 1) ? 0: 1;
				$color = ($colorcount == 1) ? $bg = $this->bg1 : $bg = $this->bg2;
				$content .= '<tr style="background-color:'.$bg.'; border-top : 1px dotted #a3a3a3;">
								<td>'.$i.'</td>
								<td>'.$key.'</td>';
				$content .= $this->createSelector ($i);
				$content .= '</tr>';
				$i++;
			}
			$content .= '</table><div align="right"><input type="submit" name="map" value="'.$LANG->getLL('f1.tab3.mapper.import').'" '.($_POST['importNow'] ? 'disabled' : '').'></div>';
			$content .= '</fieldset>';
		}
		return $content;
	}

	/**
	 * Check if all necessary mapping information is provided and if we may proceed with the import workflow.
	 *
	 * @return	string		HTML content
	 */
	function evaluateMappingForm () {

		global $LANG;

		// merge mandatory and userdefined mandatory mapping fields
		$this->mandatoryFields = array_merge($this->mandatoryFields,$this->additionalMandatoryFieldsFE);

		$this->importOK = FALSE;
//$this->importOK = TRUE;
		$mandatoryFieldError = array();

		$m = count($this->inData['fieldmap']);
		$n = array_diff($this->inData['fieldmap'], array('')); //delete empty values
		$org = $n;
		$x = count($n);
		$n = array_unique($n); // delete duplicate values

		$duplicateMapping = array_diff_assoc($org,$n);

		if (!empty($duplicateMapping)) {
			$msg = ' <b>'.implode(', ',$duplicateMapping).'</b>';
		}

		$y = count($n);
		$content = '';

		// check if we have already a mapping
		if ($y != 0) {
			$content .= '<fieldset>';

			// do we have mapping entries ???
			if (!$this->inData['import']) {
				$content .= sprintf($LANG->getLL('f1.tab3.mapper.message.info'),$x,$m,$y);
			}

			// do we have multiple mappings for a distinct field ???
			if ($x != $y) {
				$content .= ' - '.$LANG->getLL('f1.tab3.mapper.message.error').$msg.'<br>
						<legend align=left><b>'.$LANG->getLL('f1.tab3.mapper.fieldset2.error').'</b></legend>';
			} else {
				// mapping seems to be OK, continue with import button or map additional values
				if (!$this->inData['import']) {

					foreach ($this->mandatoryFields as $mandatoryField) {
						if (!in_array($mandatoryField, $this->inData['fieldmap'])) {
							$mandatoryFieldError[] = $mandatoryField;
							$this->importOK = FALSE;
						} else {
							$this->importOK = TRUE;
						}
					}

					if (!empty($mandatoryFieldError)) {
						$content .= $LANG->getLL('f1.tab3.mapper.message.provideMandatory').' <b>'.implode(', ',$mandatoryFieldError).'</b>';
					} else {
						$content .= '
								<legend align=left><b>'.$LANG->getLL('f1.tab3.mapper.fieldset2').'</b></legend>
								'.$LANG->getLL('f1.tab3.mapper.message.allMandatoryYes');
					}
					$content .= '</fieldset>';
				}

				//prepare POST data for next pageclick
				$i = 0;
				foreach ($this->CSV as $row) {
					$m = 0;
					foreach ($this->inData['fieldmap'] as $val) {
						if ($val != '') {
							$this->map[$i][$val] = $row[$m];
							$this->numMap[$val] = $m;
						}
						$m++;
					}
					$i++;
				}
			}
		}
		return $content;
	}

	/**
	 * Creates the HTML mapping form which we need to map CSV fields to DB fields. Provides examples read from CSV to support the mapping process.
	 *
	 * @param	integer		number of column for which to create a fieldmap
	 * @return	string		returns a HTML TD element
	 */
	function createSelector($x) {

		global $LANG;
		$content = '';

		if (empty($this->columnNamesFromCSV[$x])) {
			$header[$x] = "Feld" . $x;
		} else {
			$header[$x] = $this->columnNamesFromCSV[$x];
		}

		$content .= '<td>';
		$content .= '<input type="hidden" name="tx_rsuserimp[fieldname]['.$x.']" value="'.$header[$x].'" title="'.$header[$x].'">';
//		$content .= '<input type="checkbox" name="fieldcheck['.$x.']" value="'.$x.'" checked title="Check to proceed this field">';
		$content .= $this->fieldSelector($x);
		$content .= '</td>';
		$content .= '<td>';
		for ($n=0; $n <= ($this->previewNum-1); $n++) { //print CSV values in a row
			$row = $this->CSV[$n];
			$colorcount = ($colorcount == 1) ? 0: 1;
			$color = ($colorcount == 1) ? $bg=$this->bg1 : $bg=$this->bg2;
			$content .= '<div style="background-color:'.$bg.'">'.($this->CSV[$n][$x] ? $this->CSV[$n][$x].'</div>' : '&nbsp;</div>');
		}
		$content .= '</td>';
		return $content;
	}

	/**
	 * Creates a HTML select box for the mapping process.
	 *
	 * @param	integer		number of column for which to create a HTML selct box element ...
	 * @return	string		HTML SELECT element 		...
	 */
	function fieldSelector($n) {

		global $LANG;

		$box = '<select name="tx_rsuserimp[fieldmap]['.$n.']" '.$disabled.' size="1" '.($_POST['importNow'] ? 'disabled' : '').'>'."\n";
		$box .= '<option value="">'.$LANG->getLL('f1.tab3.mapper.mapsTo').'</option>'."\n";
		foreach ($this->columnNamesFromDB as $key => $value) {
			$box.='<option value="' . $value. '"';
			if ($this->inData['fieldmap'][$n] == $value) {
				$box .= ' SELECTED ';
			}
			$box .= '>' . $value. '</option>'."\n";
		}
		$box.='</select>'."\n";
		return $box;
	}

	/*************************
	 *
	 * Section 3
	 *
	 * Explanation:
	 * The final DB import process.
	 *
	 *************************/

	/**
	 * Import users from CSV after all necessary mapping info has been provided.
	 *
	 * @return	string		formated HTML to be printed
	 */
	function importUsers () {

		global $BE_USER, $LANG, $FILEMOUNTS;

		// get all FE users from DB
		$res = $GLOBALS['TYPO3_DB']->exec_SELECTquery('username','fe_users','');
		while($row = $GLOBALS['TYPO3_DB']->sql_fetch_row($res)) {
		  $users[] = $row[0];
		 }
 		$GLOBALS['TYPO3_DB']->sql_free_result($res);

		// read the import file
		$CSV = $this->readCSV();
		$n=0;
		array($main);
		foreach ($CSV as $row) {
			foreach ($this->numMap as $key => $value) {
				$sub[$key] = $row[$value];
			}
			$main[$n] = array_merge($sub, $this->defaultUserData);
			unset($sub);
			$n++;
		}

		$i = 0;
		$j = 0;
		$m = 0;
		foreach ($main as $v1) { // process users one by one
			$msg = array();
			$importUser = TRUE;

			if ($this->userType == 'FE') {
				$msg[] = $this->checkUserDataFE($v1,$importUser);
			} else if ($this->userType == 'BE') {
				$msg[] = $this->checkUserDataBE($v1,$importUser);
			} else {
				die($LANG->getLL('f1.tab5.error.unknownFatalError'));
			}

			// handle the user array: check if we have users, rename them if necessary, skip import if necessary
			if (!empty($users) && $importUser) { // if we have already users in the DB
				while (in_array($v1['username'], $users)) { // the username already exists
					if (!$this->enableAutoRename) {
						$msg = array();
						$msg[] = $LANG->getLL('f1.tab5.error.duplicateUserName');
						$importUser = FALSE;
						break 1;
					} else {
						$newName = $v1['username'].'0'; // rename current import users
						if ( strlen($newName) > 39 ) { // check for max value
							$msg = array();
							$msg[] = $LANG->getLL('f1.tab5.error.userNameTooLong');
							$importUser = FALSE;
							break 1;
						} else {
							$v1['username'] = $newName;
							$main[$i]['username'] = $v1['username'];
						}
					}
				}
			}

			if ($importUser) {
				$res = $GLOBALS['TYPO3_DB']->exec_INSERTquery('fe_users',$v1); //insert
				$uid = $GLOBALS['TYPO3_DB']->sql_insert_id(); //get uid
				$data[] = $uid;
				//	writelog($type,$action,$error,$details_nr,$details,$data,$tablename,$recuid,$recpid
				// how do you localize user-defined log messages, is this intended at all???
				$BE_USER->writelog(1,1,0,'','User %s (UID %s) created by CSV import action',Array($v1['username'],$uid,));
				$users[] = $v1['username'];
				$i++;
				$content .= '<b><font color="#76CF67">'.sprintf($LANG->getLL('f1.tab5.userInserted'),$v1['username'],$v1['name'],$uid).'</font></b> '.(!empty($msg) ? implode(',',$msg) : '').'<br />';
				$rollbackDataTemp[] = $uid;
			} else {
				$export[] = $CSV[$m];
				$content .= '<b><font color="red">'.sprintf($LANG->getLL('f1.tab5.userSkipped'),$v1['username'],$v1['name']).'</font></b>'.(!empty($msg) ? implode(',',$msg) : '').'<br />';
				$j++;
			}
		$m++;
		} //end foreach

		$content = '<b>'.sprintf($LANG->getLL('f1.tab5.usersImported'),$m,$i,$j).'</b><br>'.$content;

		if ($this->createDropFile) {
			/**
			 *	If createDropFile is set, create a drop file which holds all dropped users.
			 * The CSV format is determined by the import settings, so this file may be imported
			 * by simply using the previous preset setting.
			 */

			$fileInfo = t3lib_basicFileFunctions::getTotalFileInfo($this->file);
			//$newFileName = t3lib_div::getFileAbsFileName('uploads/rs_userimp/' . 'DROPPED_'.$fileInfo['file']);
			$newFileName = 'DROPPED_'.$fileInfo['file'];

			$fileContent = array();
			$fileContent[0][] = $this->columnNamesFromCSV;
			$fileContent = array_merge($fileContent[0],$export);

			$newAbsFile = t3lib_div::getFileAbsFileName('uploads/tx_rsuserimp/' . 'DROPPED_'.$fileInfo['file']); //PATH_site.'typo3temp/'.$newFileName;
			$newRelFile = '/uploads/tx_rsuserimp/'.$newFileName;

			_fputcsv($newAbsFile, $fileContent, $this->fieldDelimiter, $this->fieldEncaps);

			$content .='
				<div align="center"><a href="'.$newRelFile.'">'.$LANG->getLL('f1.tab5.downloadFile').'</a>
				';
		}

		if (!empty($rollbackDataTemp)) {

			$fileInfo = t3lib_basicFileFunctions::getTotalFileInfo($this->file);
			$file = $fileInfo['file'];

			$rollbackDataSets = implode(",", $rollbackDataTemp);

			$rollbackData = array (
				'user_uid' => $BE_USER->user['uid'],
				'title' => 'Import session of user '.$BE_USER->user['username'].' [UID '.$BE_USER->user['uid'].']: '.$i.' users imported.',
				'usertype' => $this->userType,
				'num_imp' => $i,
				'num_drop' => $j,
				'file' => $file,
				'dropfile' => $newAbsFile,
				'active' => '1',
				'crdate' => time(),
				'session_data' => $rollbackDataSets
			);

			$res = $GLOBALS['TYPO3_DB']->exec_INSERTquery('tx_rsuserimp_sessions',$rollbackData); //insert
		}

		/**
		 * After parsing the CSV file, the file gets deleted by either moving it to the recycler (if
		 * present) or by deleting it.
		 */
		$FILE = array();

		$FILE['delete'][]=array('data'=>$this->file);

		$fileProcessor = t3lib_div::makeInstance('t3lib_extFileFunctions');
		$fileProcessor->init($FILEMOUNTS, $TYPO3_CONF_VARS['BE']['fileExtensions']);
		$fileProcessor->init_actionPerms($BE_USER->user['fileoper_perms']);
		$fileProcessor->useRecycler = $this->useRecycler; // 0 = no, 1 = if available, 2 = always

		$fileProcessor->start($FILE);
		$fileProcessor->processData();

		return $content;
	}

	/**
	 * Each value passed is manipulated by REFERENCE.
	 * $user is the data array for the current user.
	 * $importUser is the (boolean) import flag whether to import or not import the user.
	 * The value returned is the HTML content to be displayed.
	 *
	 * @param	[type]		$$user: ...
	 * @param	[type]		$importUser: ...
	 * @return	[type]		...
	 */
	function checkUserDataFE (&$user,&$importUser) {

		global $LANG;

		/* knock out conditions - default mandatory fields */

		$importUser = TRUE;

		if ( empty($user['username']) ) { // check for empty username value
			$error[] = $LANG->getLL('f1.tab5.error.emptyUserName');//'empty username';
			$importUser = FALSE;
			$fatalError = TRUE;
		}

		if ( strlen($user['username']) > 50 ) { // check for max username value
			$error[] = $LANG->getLL('f1.tab5.error.userNameTooLong');//'username too long';
			$importUser = FALSE;
			$fatalError = TRUE;
		}

		if ( empty($user['password']) ) { // check for empty password value
			$error[] = $LANG->getLL('f1.tab5.error.emptyPassword');//'empty password';
			$importUser = FALSE;
			$fatalError = TRUE;
		}

		if ( strlen($user['password']) > 39 ) { // check for max password value
			$error[] = $LANG->getLL('f1.tab5.error.passwordTooLong');//'password too long';
			$importUser = FALSE;
			$fatalError = TRUE;
		}

		/* knock out conditions - user-defined mandatory fields */

		if (!empty($this->additionalMandatoryFieldsFE)) {
			foreach ($this->additionalMandatoryFieldsFE as $value) {
				if ( empty($user[$value]) ) {
					$error[] = sprintf($LANG->getLL('f1.tab5.error.emptyMandatory'),$value); // empty user-defined mandatory field
					$importUser = FALSE;
					$fatalError = TRUE;
				}
			}
		}

		/* These are recoverable conditions */

		if ( !$fatalError && $this->enableAutoRename ) {
			if ( strlen($user['password']) != strlen(ereg_replace(' ','',$user['password'])) ) { // check for space in password
				$msg[] = $LANG->getLL('f1.tab5.corrected.WSP');//'corrected whitespace in password';
				$user['password'] = ereg_replace(' ', '', $user['password']); // replace spaces
			}
			if ( strlen($user['username']) != strlen(ereg_replace(' ','',$user['username'])) ) { // check for space in password
				$msg[] = $LANG->getLL('f1.tab5.corrected.WSU');//'corrected whitespace in username';
				$user['username'] = ereg_replace(' ', '', $user['username']); // replace spaces
			}
		}

		/* knock out conditions if $this->enableAutoRename is NOT set*/

		if ( !$fatalError && !$this->enableAutoRename && strlen($user['username']) != strlen(ereg_replace(' ','',$user['username'])) ) { // check for space in username
			$msg[] = $LANG->getLL('f1.tab5.error.WSU'); //'whitespace in username';
			$importUser = FALSE;
		}

		if ( !$fatalError && !$this->enableAutoRename && strtolower($user['username']) != $user['username'] ) { // check for uppercase username values
			$msg[] = $LANG->getLL('f1.tab5.error.UCU');//'uppercase in username';
			$importUser = FALSE;
		}

		if ( !$fatalError && !$this->enableAutoRename && strlen($user['password']) != strlen(ereg_replace(' ','',$user['password'])) ) { // check for space in password
			$msg[] = $LANG->getLL('f1.tab5.error.WSP'); //'whitespace in password';
			$importUser = FALSE;
		}

		if ( !$fatalError && !$this->enableAutoRename && strtolower($user['password']) != $user['password'] ) { // check for uppercase password values
			$msg[] = $LANG->getLL('f1.tab5.error.UCP');//'uppercase in password';
			$importUser = FALSE;
		}

		/* These conditions may be recovered */

		if ( !$fatalError && $this->enableAutoRename ) {
			if ( strtolower($user['username']) != $user['username'] ) { // check for uppercase username values
				$msg[] = $LANG->getLL('f1.tab5.corrected.UCU'); //'corrected uppercase in username';
				$user['username'] = strtolower($user['username']);
				$importUser = TRUE;
			}

			if ( strtolower($user['password']) != $user['password'] ) { // check for uppercase password values
				$msg[] = $LANG->getLL('f1.tab5.corrected.UCP');//'corrected uppercase in password';
				$user['password'] = strtolower($user['password']);
				$importUser = TRUE;
			}
		}

		/* Generate the content string */

		if ($fatalError) {
			$content = !empty($error) ? $LANG->getLL('f1.tab5.error').implode(', ',$error) : '';
		}

		if (!empty($msg)) {
			$content = $LANG->getLL('f1.tab5.warning').implode(', ',$msg);
		}

		return $content;
	}

	/**
	 * This function does nothing. It will be used in a later version
	 * where BE user import will be enabled.
	 *
	 * @param	[type]		$$user: ...
	 * @param	[type]		$importUser: ...
	 * @return	[type]		...
	 */
	function checkUserDataBE (&$user,$importUser) {
		return;
	}

} //end class

/* General helper functions */

/**
 * In contrast to fgetcsv(), fputcsv() is not yet in the PHP 4 core (but still in PHP CSV).
 * So we fake the function here. Should be interchangeable once the official version is out.
 *
 * @param	[type]		$fileName: ...
 * @param	[type]		$dataArray: ...
 * @param	[type]		$delimiter: ...
 * @param	[type]		$enclosure: ...
 * @return	[type]		...
 */
function _fputcsv($fileName, $dataArray, $delimiter, $enclosure){

	// Build the string
	$string = "";
	$writeDelimiter = FALSE;
	foreach($dataArray as $dataRow){
		foreach ($dataRow as $dataElement) {
			if($writeDelimiter) $string .= $delimiter;
			$string .= $enclosure . $dataElement . $enclosure;
			$writeDelimiter = TRUE;
		} // end foreach($dataArray as $dataElement)
	// Append new line
	$string .= "\n";
	}
	//	t3lib_div::writeFileToTypo3tempDir($fileName,$string);
	if (t3lib_div::writeFile($fileName,$string)) {
		//	print_r('success');
	} else {
		//	print_r('error');
	}
}

if (defined("TYPO3_MODE") && $TYPO3_CONF_VARS[TYPO3_MODE]["XCLASS"]["ext/rs_userimp/mod1/class.tx_rsuserimp.php"])	{
	include_once($TYPO3_CONF_VARS[TYPO3_MODE]["XCLASS"]["ext/rs_userimp/mod1/class.tx_rsuserimp.php"]);
}
?>